
# q-expansion of newform 176.4.a.i, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 176
weight = 4
poly_data = [-3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 4], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [1, 1], [1, 2], [-10, 1], [11, 0], [40, -5], [-62, 3], [-36, -15], [49, 9], [72, -14], [17, -7], [27, -2], [268, -1], [30, 4], [136, 30], [-246, -14], [-317, 33], [420, 46], [-377, 5], [339, -19], [-200, -117], [-158, -164], [-234, -30], [-921, -82], [1097, 36], [566, -105], [880, -122], [-174, -95], [490, 80], [-855, 166], [604, 103], [222, 199], [905, 102], [1466, -207], [530, -427], [1870, 102], [1409, 154], [2060, 96], [-1608, -163], [2058, 36], [2871, -225], [1303, -304], [-2321, 87], [-572, 276], [-4322, 116], [-408, -60], [-1500, 232], [1973, 283], [-1422, 461], [3495, -228], [2304, 302], [-114, 606], [1456, 354], [-409, -99], [-1226, 290], [-3194, -431], [3010, -292], [-3032, -223], [166, 58], [198, 738], [4964, -34], [-3276, 809], [1588, 13], [1956, -798], [3263, 566], [109, -18], [-3607, 711], [5726, -861], [632, 755], [5106, -233], [-2589, -1202], [7808, 533], [-8215, 209], [-2726, -373], [1829, -143], [4585, -253], [-2847, 860], [9262, 714], [-2950, -478], [2514, -968], [12100, -406], [-170, 1922], [-2658, -325], [-6755, -720], [2000, 1418], [-10885, 113], [8315, -1250], [-13380, 406], [6660, -278], [2243, -819], [5019, -1463], [-10784, -113], [5641, 1831], [10472, -412], [3580, -2380], [-8974, -172], [2487, 572], [-5859, -1944], [-9356, 447], [-2348, -1683], [7984, -438], [-7910, 593], [9332, 82], [-2472, 1126], [7180, 1412], [-17607, 306], [-484, -1530], [3740, 88], [-11632, -224], [-6994, -1699], [-9650, -1764], [4592, -1172], [-3246, -2812], [4393, 2461], [-8791, -1839], [6375, 1988], [-8189, -2985], [6031, -641], [13183, 988], [-2338, 1856], [2797, 72], [11006, -1760], [5538, 1109], [21688, -1134], [-8993, 2715], [5126, 3583], [15215, -564], [15393, 2487], [4029, -509], [10308, -242], [22750, -634], [9020, -2916], [-3937, -1847], [5666, -4340], [21432, -1871], [10616, 3207], [-24126, -1652], [-5656, -1454], [-9571, 2520], [14772, -1281], [-23842, 1392], [-8618, -1620], [-1887, 4343], [1864, -5181], [1897, 438], [37781, 59], [10238, 4724], [13064, -4380], [-19425, 1459], [-2352, 1732], [-11760, 4545], [11645, 1130], [-48940, -22], [-25394, 2324], [-29652, -1160], [-18348, -2122], [-4066, 3928], [-8310, -2146], [-29524, -729], [28626, 1906], [2955, -4357], [10102, 4451], [27176, 2742], [12865, -1113], [12117, -350], [-1815, -4489], [4304, -4876], [-29374, -371]]
